<?php
/**
 * Component-View Module Entry Point
 * 
 * @package    Component-View
 * @subpackage mod_comview
 * @version    1.0.0
 *
 * @author     Manuel Haeusler <tech.spuur@quickline.com>
 * @copyright  2018 Manuel Haeusler
 * @license    GNU/GPL, see LICENSE.php
 *
 * mod_comview is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */

// No direct access
defined('_JEXEC') or die;

$basePath = dirname(__FILE__);

// Guess the field context based on view.
$jinput = JFactory::getApplication()->input;
$context = $jinput->get('option') . '.' . $jinput->get('view');

// Get the Module-Params
$params_type = json_decode(base64_decode($params->get('type')));
$params_request = $params_type->request;

$params_title = $params->get('title');
$params_isTitle = $params->get('istitle');
$params_isButton = $params->get('isbutton');
$params_txtButton = $params->get('txtbutton');
$params_settings = $params->get('menuitem');

// Load the language file of the component
$base_dir = JPATH_SITE . '/language';
$lang = JFactory::getLanguage();
$lang->load($params_request->option, JPATH_SITE, $lang->getTag(), true);

// Get the additional settings (specials)
$params_specials = (object) array('category_id'=>$params->get('category_id'),'element_id'=>$params->get('element_id'),'parentelement_id'=>$params->get('parentelement_id'),'tags'=>$params->get('tags'),'types'=>$params->get('types'),'language'=>$params->get('language'),'searchstring'=>$params->get('searchstring'));
// Definition, which special is used in which component view
$specials = (object) array('category_id'=>(object) array('com_content'=>['categories','archive','category'],'com_contact'=>['categories','category'],'com_newsfeeds'=>['categories','category']),'element_id'=>(object) array('com_content'=>['article'],'com_contact'=>['contact'],'com_newsfeeds'=>['newsfeed'],'com_finder'=>['search']),'parentelement_id'=>(object) array('com_tags'=>['tags']),'tags'=>(object) array('com_content'=>['category'],'com_tags'=>['tag']),'types'=>(object) array('com_tags'=>['tag']),'language'=>(object) array('com_tags'=>['tag','tags']),'searchstring'=>(object) array('com_search'=>['search'],'com_finder'=>['search']));
// Definition, how to transmit the specials in the POST request
$specials_form = (object) array('category_id'=>array('com_content'=>'id','com_contact'=>'id','com_newsfeeds'=>'id'), 'element_id'=>array('com_content'=>'id','com_contact'=>'id','com_newsfeeds'=>'id','com_finder'=>'f'), 'parentelement_id'=>array('com_tags'=>'parent_id'), 'tags'=>array('com_content'=>'filter_tag[0]','com_tags'=>'id[0]'), 'types'=>array('com_tags'=>'types[0]'), 'language'=>array('com_tags'=>'tag_list_language_filter'), 'searchstring'=>array('com_search'=>'searchword','com_finder'=>'q'));

$whichSpecials = function ($option, $comview) use($specials) { 
    $array = array();
    foreach ($specials as $param => $com) {
        foreach ($com as $component => $views) {
            if ($component == $option && in_array($comview, $views)) {
                if ($component == 'com_content' && $views == 'category' && $param == 'tags') {
                    if($params_request->layout == 'blog') {array_push($array, $param);}
                } else {
                    array_push($array, $param);
                }                
            }
        }
    }
    return $array;
};


$link = 'index.php';

?>
<?php if ($params_isTitle == 1): ?>
    <h3><?php echo JText::_($params_title) ;?></h3>
<?php endif ?>
<?php if ($params_isButton == 1): ?>
    <button class="btn" id="submit"><?php echo $params_txtButton ;?></button>
    <br />
    <br />
<?php endif ?>
<div id="mod_comview-content"></div>
<?php
// echo '<br /><br /><div><h6>Debug:</h6><p>Request-Parameter: ' . print_r($params_request) . '</p></div>';
?>


<script type="text/javascript">
<?php if ($params_isButton == 1): ?>
jQuery("#submit").click(function(event){
<?php endif ?>

    // create form data for POST
    var formData = new FormData();
    <?php foreach ($params_request as $param => $param_value): ?>
        formData.append('<?php echo $param; ?>', '<?php echo $param_value; ?>');
    <?php endforeach ?>
    <?php try {
        if (is_null($params_request->layout)) {
            echo "formData.append('layout', 'default');";
        }
    } catch (Exception $e) {echo "formData.append('layout', 'default'); console.log('layout not defined in request');";};
    ?>
    formData.append('tmpl', 'component');
    <?php if ($params_settings > 0 ): ?>
        formData.append('Itemid', '<?php echo $params_settings; ?>');
    <?php endif ?>
    formData.append('[token]', '1');

    // Spezialfälle
    <?php foreach ($whichSpecials($params_request->option, $params_request->view) as $key => $value): ?>
        <?php if (is_array($params_specials->$value) && strpos($specials_form->$value[$params_request->option], '[0]') !== false): ?>
            <?php foreach ($params_specials->$value as $k => $val): ?>
                <?php $i = 0; ?>
                formData.append('<?php echo str_replace("[0]", "[".$i."]", $specials_form->$value[$params_request->option]); ?>', '<?php echo $val; ?>');
                <?php $i = $i + 1; ?>
            <?php endforeach ?>            
        <?php else: ?>
            formData.append('<?php echo $specials_form->$value[$params_request->option]; ?>', '<?php echo $params_specials->$value; ?>');
        <?php endif ?>
    <?php endforeach ?>

    // Insert Itemid in URL 
    <?php 
    if ($params_settings > 0 ) {
        $link = $link . '?Itemid=' . $params_settings;
    }
    ?>

    // Ajax request for check if user already passed a call  JRoute::_($link)
    var xhr=new XMLHttpRequest();
    xhr.open("POST", "<?php echo $link; ?>", true);
    xhr.setRequestHeader('X-Requested-With', '');

    xhr.onreadystatechange = function() {
        if (this.readyState == 4 && this.status == 200) {

            // grab relevant css and script from <head>
            function getRelevantHead(txt, key) { 
                txt = txt.replace(/(\r\n|\n|\r)/gm, "");
                // serach key in <head> and get indices
                var pos = 0, i = -1, pos_indices = [];
                while (pos != -1) {
                    pos = txt.indexOf(key, i+1);
                    if (pos != 0 && pos != -1 && pos < txt.indexOf("/head>")) {
                        pos_indices.push(pos);
                    };
                    i = pos;
                }
                // loop over indices and create the relevant css and script
                //var pos = txt.indexOf(key);
                var head = '';
                pos_indices.forEach(function(pos) {
                    var left = txt.slice(0, pos + 1).search(/<link(?!.*<link)|<script(?!.*<script)/g);
                    var right = txt.slice(pos + 1).search(/ \/>|<\/script>/g) + pos + 1;
                    if (txt.slice(left, right).includes("<script")) {
                        line = txt.slice(left, right) + "<\/script>";
                    } else {
                        line = txt.slice(left, right) + " \/>";
                    };
                    head = head + line + '\n';
                });                
                return head;
            };

            // delete <head> from response
            var start_pos = xhr.responseText.indexOf("/head>") + 6;
            var xhrTxt = xhr.responseText.slice(start_pos, -8);

            // insert the relevant css and scripts
            //xhrTxt = getRelevantHead(xhr.responseText,'/templates/') + xhrTxt;
            xhrTxt = getRelevantHead(xhr.responseText,'<?php echo $params_request->option; ?>') + xhrTxt; 

            // insert response to html
            jQuery("#mod_comview-content").html(xhrTxt);

            <?php if ($params_isButton == 1): ?>
            // hide Button
            jQuery("#submit").addClass("hidden");
            <?php endif ?>

            // makes Bootstrap Collapse work again
            jQuery('[data-toggle="collapse"]').click(function(e){
              e.preventDefault();
              var target_element = jQuery(this).attr("data-target");
              jQuery(target_element).collapse('toggle');
              return false;
            });

        } else if (this.status == 403) {
            alert("Response from server: 403 - Access denied");
        }
    }
    xhr.send(formData);

<?php if ($params_isButton == 1): ?>
});
<?php endif ?>

</script>